@@ -18,7 +18,7 @@
 // Copyright (C) 2009 Michael K. Johnson <a1237@danlj.org>
 // Copyright (C) 2009 Shen Liang <shenzhuxi@gmail.com>
 // Copyright (C) 2009 Stefan Thomas <thomas@eload24.com>
-// Copyright (C) 2009-2011, 2015, 2018-2020 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2009-2011, 2015, 2018-2021 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010, 2012, 2017 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2010 Jonathan Liu <net147@gmail.com>
@@ -29,11 +29,13 @@
 // Copyright (C) 2015 William Bader <williambader@hotmail.com>
 // Copyright (C) 2018 Martin Packman <gzlist@googlemail.com>
 // Copyright (C) 2019 Yves-Gaël Chény <gitlab@r0b0t.fr>
-// Copyright (C) 2019, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2019-2021 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2019 <corentinf@free.fr>
 // Copyright (C) 2019 Kris Jurka <jurka@ejurka.com>
 // Copyright (C) 2019 Sébastien Berthier <s.berthier@bee-buzziness.com>
 // Copyright (C) 2020 Stéfan van der Walt <sjvdwalt@gmail.com>
+// Copyright (C) 2020 Philipp Knechtges <philipp-dev@knechtges.com>
+// Copyright (C) 2021 Diogo Kollross <diogoko@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -61,6 +63,7 @@
 #include "SplashOutputDev.h"
 #include "Win32Console.h"
 #include "numberofcharacters.h"
+#include "sanitychecks.h"
 
 // Uncomment to build pdftoppm with pthreads
 // You may also have to change the buildsystem to
@@ -102,6 +105,12 @@ static bool gray = false;
 #ifdef USE_CMS
 static GooString displayprofilename;
 static GfxLCMSProfilePtr displayprofile;
+static GooString defaultgrayprofilename;
+static GfxLCMSProfilePtr defaultgrayprofile;
+static GooString defaultrgbprofilename;
+static GfxLCMSProfilePtr defaultrgbprofile;
+static GooString defaultcmykprofilename;
+static GfxLCMSProfilePtr defaultcmykprofile;
 #endif
 static char sep[2] = "-";
 static bool forceNum = false;
@@ -129,6 +138,7 @@ static SplashThinLineMode thinLineMode = splashThinLineDefault;
 static int numberOfJobs = 1;
 #endif // UTILS_USE_PTHREADS
 static bool quiet = false;
+static bool progress = false;
 static bool printVersion = false;
 static bool printHelp = false;
 
@@ -158,6 +168,9 @@ static const ArgDesc argDesc[] = { { "-f", argInt, &firstPage, 0, "first page to
                                    { "-gray", argFlag, &gray, 0, "generate a grayscale PGM file" },
 #ifdef USE_CMS
                                    { "-displayprofile", argGooString, &displayprofilename, 0, "ICC color profile to use as the display profile" },
+                                   { "-defaultgrayprofile", argGooString, &defaultgrayprofilename, 0, "ICC color profile to use as the DefaultGray color space" },
+                                   { "-defaultrgbprofile", argGooString, &defaultrgbprofilename, 0, "ICC color profile to use as the DefaultRGB color space" },
+                                   { "-defaultcmykprofile", argGooString, &defaultcmykprofilename, 0, "ICC color profile to use as the DefaultCMYK color space" },
 #endif
                                    { "-sep", argString, sep, sizeof(sep), "single character separator between name and page number, default - " },
                                    { "-forcenum", argFlag, &forceNum, 0, "force page number even if there is only one page " },
@@ -188,6 +201,7 @@ static const ArgDesc argDesc[] = { { "-f", argInt, &firstPage, 0, "first page to
 #endif // UTILS_USE_PTHREADS
 
                                    { "-q", argFlag, &quiet, 0, "don't print any messages or errors" },
+                                   { "-progress", argFlag, &progress, 0, "print progress info" },
                                    { "-v", argFlag, &printVersion, 0, "print copyright and version info" },
                                    { "-h", argFlag, &printHelp, 0, "print usage information" },
                                    { "-help", argFlag, &printHelp, 0, "print usage information" },
@@ -195,6 +209,8 @@ static const ArgDesc argDesc[] = { { "-f", argInt, &firstPage, 0, "first page to
                                    { "-?", argFlag, &printHelp, 0, "print usage information" },
                                    {} };
 
+static constexpr int kOtherError = 99;
+
 static bool needToRotate(int angle)
 {
     return (angle == 90) || (angle == 270);
@@ -312,6 +328,10 @@ static void savePageSlice(PDFDoc *doc, SplashOutputDev *splashOut, int pg, int x
             bitmap->writePNMFile(stdout);
         }
     }
+
+    if (progress) {
+        fprintf(stderr, "%d %d %s\n", pg, lastPage, ppmFile != nullptr ? ppmFile : "");
+    }
 }
 
 #ifdef UTILS_USE_PTHREADS
@@ -353,10 +373,13 @@ static void processPageJobs()
         splashOut->setEnableFreeType(enableFreeType);
 #    ifdef USE_CMS
         splashOut->setDisplayProfile(displayprofile);
+        splashOut->setDefaultGrayProfile(defaultgrayprofile);
+        splashOut->setDefaultRGBProfile(defaultrgbprofile);
+        splashOut->setDefaultCMYKProfile(defaultcmykprofile);
 #    endif
         splashOut->startDoc(pageJob.doc);
 
-        savePageSlice(pageJob.doc, splashOut, pageJob.pg, x, y, w, h, pageJob.pg_w, pageJob.pg_h, pageJob.ppmFile);
+        savePageSlice(pageJob.doc, splashOut, pageJob.pg, param_x, param_y, param_w, param_h, pageJob.pg_w, pageJob.pg_h, pageJob.ppmFile);
 
         delete splashOut;
         delete[] pageJob.ppmFile;
@@ -367,7 +390,6 @@ static void processPageJobs()
 
 int main(int argc, char *argv[])
 {
-    PDFDoc *doc;
     GooString *fileName = nullptr;
     char *ppmRoot = nullptr;
     char *ppmFile;
@@ -379,15 +401,13 @@ int main(int argc, char *argv[])
     pthread_t *jobs;
 #endif // UTILS_USE_PTHREADS
     bool ok;
-    int exitCode;
     int pg, pg_num_len;
     double pg_w, pg_h;
 #ifdef USE_CMS
-    cmsColorSpaceSignature displayprofilecolorspace;
+    cmsColorSpaceSignature profilecolorspace;
 #endif
 
     Win32Console win32Console(&argc, &argv);
-    exitCode = 99;
 
     // parse args
     ok = parseArgs(argDesc, &argc, argv);
@@ -405,9 +425,11 @@ int main(int argc, char *argv[])
         if (!printVersion) {
             printUsage("pdftoppm", "[PDF-file [PPM-file-prefix]]", argDesc);
         }
-        if (printVersion || printHelp)
-            exitCode = 0;
-        goto err0;
+        if (printVersion || printHelp) {
+            return 0;
+        } else {
+            return kOtherError;
+        }
     }
     if (argc > 1)
         fileName = new GooString(argv[1]);
@@ -470,7 +492,7 @@ int main(int argc, char *argv[])
         delete fileName;
         fileName = new GooString("fd://0");
     }
-    doc = PDFDocFactory().createPDFDoc(*fileName, ownerPW, userPW);
+    std::unique_ptr<PDFDoc> doc(PDFDocFactory().createPDFDoc(*fileName, ownerPW, userPW));
     delete fileName;
 
     if (userPW) {
@@ -480,8 +502,7 @@ int main(int argc, char *argv[])
         delete ownerPW;
     }
     if (!doc->isOk()) {
-        exitCode = 1;
-        goto err1;
+        return 1;
     }
 
     // get page range
@@ -493,7 +514,7 @@ int main(int argc, char *argv[])
         lastPage = doc->getNumPages();
     if (lastPage < firstPage) {
         fprintf(stderr, "Wrong page range given: the first page (%d) can not be after the last page (%d).\n", firstPage, lastPage);
-        goto err1;
+        return kOtherError;
     }
 
     // If our page range selection and document size indicate we're only
@@ -501,7 +522,7 @@ int main(int argc, char *argv[])
     // filter out that single page.
     if (firstPage == lastPage && ((printOnlyEven && firstPage % 2 == 1) || (printOnlyOdd && firstPage % 2 == 0))) {
         fprintf(stderr, "Invalid even/odd page selection, no pages match criteria.\n");
-        goto err1;
+        return kOtherError;
     }
 
     if (singleFile && firstPage < lastPage) {
@@ -526,28 +547,49 @@ int main(int argc, char *argv[])
         displayprofile = make_GfxLCMSProfilePtr(cmsOpenProfileFromFile(displayprofilename.c_str(), "r"));
         if (!displayprofile) {
             fprintf(stderr, "Could not open the ICC profile \"%s\".\n", displayprofilename.c_str());
-            goto err1;
+            return kOtherError;
         }
         if (!cmsIsIntentSupported(displayprofile.get(), INTENT_RELATIVE_COLORIMETRIC, LCMS_USED_AS_OUTPUT) && !cmsIsIntentSupported(displayprofile.get(), INTENT_ABSOLUTE_COLORIMETRIC, LCMS_USED_AS_OUTPUT)
             && !cmsIsIntentSupported(displayprofile.get(), INTENT_SATURATION, LCMS_USED_AS_OUTPUT) && !cmsIsIntentSupported(displayprofile.get(), INTENT_PERCEPTUAL, LCMS_USED_AS_OUTPUT)) {
             fprintf(stderr, "ICC profile \"%s\" is not an output profile.\n", displayprofilename.c_str());
-            goto err1;
+            return kOtherError;
         }
-        displayprofilecolorspace = cmsGetColorSpace(displayprofile.get());
+        profilecolorspace = cmsGetColorSpace(displayprofile.get());
+        // Note: In contrast to pdftops we do not fail if a non-matching ICC profile is supplied.
+        //       Doing so would be pretentious, since SplashOutputDev by default assumes sRGB, even for
+        //       the CMYK and Mono cases.
         if (jpegcmyk || overprint) {
-            if (displayprofilecolorspace != cmsSigCmykData) {
+            if (profilecolorspace != cmsSigCmykData) {
                 fprintf(stderr, "Warning: Supplied ICC profile \"%s\" is not a CMYK profile.\n", displayprofilename.c_str());
             }
         } else if (mono || gray) {
-            if (displayprofilecolorspace != cmsSigGrayData) {
+            if (profilecolorspace != cmsSigGrayData) {
                 fprintf(stderr, "Warning: Supplied ICC profile \"%s\" is not a monochrome profile.\n", displayprofilename.c_str());
             }
         } else {
-            if (displayprofilecolorspace != cmsSigRgbData) {
+            if (profilecolorspace != cmsSigRgbData) {
                 fprintf(stderr, "Warning: Supplied ICC profile \"%s\" is not a RGB profile.\n", displayprofilename.c_str());
             }
         }
     }
+    if (!defaultgrayprofilename.toStr().empty()) {
+        defaultgrayprofile = make_GfxLCMSProfilePtr(cmsOpenProfileFromFile(defaultgrayprofilename.c_str(), "r"));
+        if (!checkICCProfile(defaultgrayprofile, defaultgrayprofilename.c_str(), LCMS_USED_AS_INPUT, cmsSigGrayData)) {
+            return kOtherError;
+        }
+    }
+    if (!defaultrgbprofilename.toStr().empty()) {
+        defaultrgbprofile = make_GfxLCMSProfilePtr(cmsOpenProfileFromFile(defaultrgbprofilename.c_str(), "r"));
+        if (!checkICCProfile(defaultrgbprofile, defaultrgbprofilename.c_str(), LCMS_USED_AS_INPUT, cmsSigRgbData)) {
+            return kOtherError;
+        }
+    }
+    if (!defaultcmykprofilename.toStr().empty()) {
+        defaultcmykprofile = make_GfxLCMSProfilePtr(cmsOpenProfileFromFile(defaultcmykprofilename.c_str(), "r"));
+        if (!checkICCProfile(defaultcmykprofile, defaultcmykprofilename.c_str(), LCMS_USED_AS_INPUT, cmsSigCmykData)) {
+            return kOtherError;
+        }
+    }
 #endif
 
 #ifndef UTILS_USE_PTHREADS
@@ -559,8 +601,11 @@ int main(int argc, char *argv[])
     splashOut->setEnableFreeType(enableFreeType);
 #    ifdef USE_CMS
     splashOut->setDisplayProfile(displayprofile);
+    splashOut->setDefaultGrayProfile(defaultgrayprofile);
+    splashOut->setDefaultRGBProfile(defaultrgbprofile);
+    splashOut->setDefaultCMYKProfile(defaultcmykprofile);
 #    endif
-    splashOut->startDoc(doc);
+    splashOut->startDoc(doc.get());
 
 #endif // UTILS_USE_PTHREADS
 
@@ -583,23 +628,34 @@ int main(int argc, char *argv[])
         if (scaleDimensionBeforeRotation && needToRotate(doc->getPageRotate(pg)))
             std::swap(pg_w, pg_h);
 
+        // Handle requests for specific image size
         if (scaleTo != 0) {
             resolution = (72.0 * scaleTo) / (pg_w > pg_h ? pg_w : pg_h);
             x_resolution = y_resolution = resolution;
+            pg_w = pg_h = scaleTo;
         } else {
             if (x_scaleTo > 0) {
                 x_resolution = (72.0 * x_scaleTo) / pg_w;
+                pg_w = x_scaleTo;
                 if (y_scaleTo == -1)
                     y_resolution = x_resolution;
             }
+
             if (y_scaleTo > 0) {
                 y_resolution = (72.0 * y_scaleTo) / pg_h;
+                pg_h = y_scaleTo;
                 if (x_scaleTo == -1)
                     x_resolution = y_resolution;
             }
+
+            // No specific image size requested---compute the size from the resolution
+            if (x_scaleTo <= 0) {
+                pg_w = pg_w * (x_resolution / 72.0);
+            }
+            if (y_scaleTo <= 0) {
+                pg_h = pg_h * (y_resolution / 72.0);
+            }
         }
-        pg_w = pg_w * (x_resolution / 72.0);
-        pg_h = pg_h * (y_resolution / 72.0);
 
         if (!scaleDimensionBeforeRotation && needToRotate(doc->getPageRotate(pg)))
             std::swap(pg_w, pg_h);
@@ -618,13 +674,13 @@ int main(int argc, char *argv[])
         }
 #ifndef UTILS_USE_PTHREADS
         // process job in main thread
-        savePageSlice(doc, splashOut, pg, param_x, param_y, param_w, param_h, pg_w, pg_h, ppmFile);
+        savePageSlice(doc.get(), splashOut, pg, param_x, param_y, param_w, param_h, pg_w, pg_h, ppmFile);
 
         delete[] ppmFile;
 #else
 
         // queue job for worker threads
-        PageJob pageJob = { .doc = doc,
+        PageJob pageJob = { .doc = doc.get(),
                             .pg = pg,
 
                             .pg_w = pg_w,
@@ -663,12 +719,5 @@ int main(int argc, char *argv[])
 
 #endif // UTILS_USE_PTHREADS
 
-    exitCode = 0;
-
-    // clean up
-err1:
-    delete doc;
-err0:
-
-    return exitCode;
+    return 0;
 }
